
import java.util.*;
/**
 * Denne klassen skal vre nodeobjektet!  
 * Klientsiden av noden! 				  
 * 
 * @author Morten Tvenning
 * @version 25 feb
 */

public class NodeKlient {
	
	
	/////////////////////////////////////////////////////
	/// ClassLocal Variables       //////////////////////	
	/////////////////////////////////////////////////////
	
	String myName; // the name of the node
	ArrayList routingTable; // list of other nodes this node knows exists
	NodeTjener server; // the server that is bound to this client
	int minmillisekunddelay;
	NodeKlient pathNodeToServer;
	int pathNodeToServerDelay;
	
	/////////////////////////////////////////////////////
	/// Constructors               //////////////////////	
	/////////////////////////////////////////////////////	
	
	/** 
	 * 
	 * Constructor som tar inn bare ett navn. 
	 * Setter igang rutingtabellen, navn
	 * pathNodeToServer og pathNodeToServerDelay. 
	 * 
	 * @param mittNavn navnet til noden som en String
	 */
	public NodeKlient(String mittNavn){
		
		routingTable = new ArrayList(); 
		this.myName = mittNavn; 	
		
		System.out.println("Instansiert med bare navn.. navnet er : " + mittNavn); 
		server = new NodeTjener(this); 
		pathNodeToServer = null;
		pathNodeToServerDelay = -10;
	}
	
	/** 
	 * Construktor som tar i mot frste routingNode og nodenavn. 
	 * Setter igang rutingtabellen, navn
	 * pathNodeToServer og pathNodeToServerDelay. 
	 * 
	 * @param innNodeRouting the first node to be added to the routingTable
	 * @param mittNavn  streng som inneholder navnet til noden.
	 */
	public NodeKlient(NodeKlient innNodeRouting, String mittNavn){
		
		routingTable = new ArrayList();
		routingTable.add(innNodeRouting);
		this.myName = mittNavn;
		//Kommentar til konsoll
		/*System.out.println("Instansiert med en routingnode og navn" + 
				"\n" + "Routingnode er : " +(String)innNodeRouting.getName() + 
				"\n" + "navnet p noden er : " + this.myName);*/
		server = new NodeTjener(this);
		pathNodeToServer = null;
		pathNodeToServerDelay = -10;
	}
	
	/** Konstructor med bare frste routingnode
	 * 
	 * Denne konstruktoren oppretter routingtable
	 * i tillegg addes frste routingnode
	 * 
	 * @param innNodeRouting the first node to be added to the routingTable
	 */
	public NodeKlient(NodeKlient innNodeRouting){
		
		// indexWordList = new ArrayList();
		routingTable = new ArrayList(); 
		routingTable.add(innNodeRouting); 
		// Kommentar til konsoll
		System.out.println("Instansiert med frste routingNode");
		server = new NodeTjener(this);
		pathNodeToServer = null;
		pathNodeToServerDelay = -10;
	}
	
	
	
	////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////
	//// 																/////
	////				Klasse Metoder! 								/////
	////																/////
	/////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////
	
	
	/** 
 	 * 
	 * Legger en nabo til i routingTabellen. 
	 */
	public void leggTilNabo(NodeKlient innnode){
		
		this.routingTable.add(innnode); 
		// System.out.println("Adding to the routingTable: " +(String)innnode.getName());
		
	} 
	
	public NodeKlient getPathNodeTilTjener(){
		return pathNodeToServer;
	}
	/**  
	 * Returnerer navnet til denne noden. 
	 * 
	 * @return Navn the name of the node
	 */
	public String getNavn(){
		
		return this.myName;
	}
	
	/** 
	 * 
	 * Returnerer frste noden i routingTabellen
	 * 
	 * @return firstNode frste noden i routingTabellen. 
	 */
	public String faaForsteObjekt(){
		
		return((String)routingTable.get(0));
			
	} 
	
	
	/**
	 * 
	 * 
	 * Returnerer denne nodens routingTabell som en ArrayList
	 * 
	 * 
	 * @return routingTable routingTabellen for noden. En tom ArrayList om noden ikke har noen routingtabell.
	 */
	public ArrayList getRoutingTable(){
		
		return this.routingTable; 
		
	}
	
	/**
	 * Returnerer tjenernoden som denne noden er koblet til. 
	 * 
	 * @return nodeServer tjenernoden som er knyttet/assosiert med denne klienten
	 */
	public NodeTjener getTjenerNode(){
		
		return this.server;
		}
	
	/** 
	 * 
	 * Sjekker routingtabellen om noden allrede finnes. 
	 * 
	 * @param node inputnoden du vil sjekke om finnes i routingtabellen
	 * @return exists true om den finnes, false om den ikke finnes i routingtabellen.  
	 */
	public boolean nodeFinnesAlleredeIRT(NodeKlient node){
		
		boolean exists = false;
		int teller=0;
		while(teller < routingTable.size()){
			if(node==((NodeKlient)routingTable.get(teller))){
				exists = true; 
				// System.out.println("Positive for the node already exists"); 
			}
			teller++;  
		}
		return exists; 
		}
	/*** 
	 * 
	 * Skriver ut stringinnholdet for nodene i routingtabellen til konsoll. 
	 *
	 */
	public void printRTtilKonsoll(){
		
		System.out.println("\n" +"***********************************************" +
				"\n" + "Printing randomized routingTable" +
				"\n" + "For --->  " + this.getNavn() +
				"\n" + "***********************************************"); 
		
		int size = routingTable.size();
		int teller=0;
		while(teller<size){
			System.out.println(((NodeKlient)routingTable.get(teller)).getNavn()); 
			teller++;
		}
		
		
	}
	
	/**
	 * Skriver ut nodens path til tjener og latency
	 *
	 */
	public void printTjenerListe(){
		
		System.out.println("\n" +"***********************************************" +
				"\n" + "Printing randomized routingTable" +
				"\n" + "For --->  " + this.getNavn() +
				"\n" + "Path to node    ---> " + this.pathNodeToServer.getNavn() +
				"\n" + "Delay to Server ---> " + this.pathNodeToServerDelay +
				"\n" + "***********************************************"); 
		
		
		
		
	}
	/**
	 * Denne metoden sender videre indexlisten om dens tjener ikke er aktiv, og lagrer 
	 * listen i nodeTjener om den er aktiv. 
	 * 
	 * @param insertList ArrayList som innholder Indextermer og fil/key/lok
	 */
	public void leggTilIndeks(ArrayList insertList){
		if(this.server.erAktiv()){
			this.server.leggtilIndeksBaseFraArrayList(insertList);
			// System.out.println("nodeClient: addToIndex done in " + this.getName());
			}
		else{
			// forwarde til en den node som er nermest server. 
			
			
		}
	}
	/**  
	 * Denne metoden setter hvor mange millisekund latency det er fra denne noden.
	 * @param delay en fiktiv verdi mellom 0 og 100 som skal representere tidsbruken ved  overfre til denne noden.
	 */
	public void setLatency(int delay){
		minmillisekunddelay = delay; 
	}
	
	/**
	 * Returnerer hvor langt det er til nermeste server. 
	 * 
	 * @return pathNodeToServerDelay. Den verdien som er lagt som minste rute til nermeste server. 
	 */
	public int getPathToTjenerLatency(){ return pathNodeToServerDelay; }
	/**
	 * Denne metoden retunerer delayen denne noden har. 
	 * @return returnerer delayen denne noden har i millisekunder (fra 1 - 100)
	 */
	public int getLatency(){ return minmillisekunddelay; }
	
	/**
	 * Sender ut en rekursiv ping om hvor nerme denne noden er en tjener.
	 * 
	 * @param hopstolive hvor mange hopp som er igjen
	 * @param innNode noden som kaller denne funksjonen til denne noden
	 * @param delay hvor mye delay som allerede finnes til server. 
	 */
	public void goTjenerPing(int hopstolive, NodeKlient innNode, int delay){
		if(hopstolive>0){
		for(int i=0; i<routingTable.size(); i++){
			if((NodeKlient)routingTable.get(i)!=innNode){
				
				((NodeKlient)routingTable.get(i)).getTjenerPing(this, hopstolive, delay);
				
			}
			
		}}
	}
	/**
	 * Denne metoden sjekker om en tjener ligger nermere eller er lenger i fra. 
	 * I "semirekursivt samarbeide med goTjenerPing"
	 * 
	 * @param innNode noden som er nermest en tjener til en hver tid
	 * @param hopstolive hvor mange hopp en tjenerping skal vare
	 * @param delay den sammenlagte delayen fra tjener.
	 */
	public void getTjenerPing(NodeKlient innNode, int hopstolive, int delay){
		
		// satt til -10 om den ikke er initiert annet enn av konstruktor
		if(this.pathNodeToServerDelay == -10){
			
			pathNodeToServer = innNode; pathNodeToServerDelay = delay;
			this.goTjenerPing(hopstolive-1, innNode, (delay+minmillisekunddelay));
			
		}
		// enten lagre som nermeste og s sende videre
		else{
			if(pathNodeToServerDelay > delay){
				// System.out.println("Har forandret til ny server");
				pathNodeToServer = innNode; pathNodeToServerDelay = delay;
				this.goTjenerPing(hopstolive-1, innNode,(delay+minmillisekunddelay));
				
			}
			// eller bare sende ping videre til routinglisten. 
			else{
				
				this.goTjenerPing(hopstolive-1, innNode, (delay+minmillisekunddelay));
				
			}
		}
	}
	/**
	 * Denne funksjonen lagrer enten listen til tjeneren sin
	 * eller sender den videre til den nermeste tjener. 
	 * 
	 * @param invarray invertert liste med name/key/lok as first element
	 */
	public void faaInverertIndeksTillegg(ArrayList invarray){
		
		if(this.server.erAktiv()){
			//System.out.println(this.getName());
			//System.out.println("Legger array til i min server");
			this.server.leggtilIndeksBaseFraArrayList(invarray);
			
			
		}
		else{
			// System.out.println("Sender array videre nermere serveren"  + "/n" +
			// 					"Delay to server: " + this.pathNodeToServerDelay);
			pathNodeToServer.faaInverertIndeksTillegg(invarray);
			
		}
	}
	
	/**
	 * Metode som rekursivt sker til nermeste tjener og fr svar tilbake i form av en 
	 * Arrayliste med strenger. Om noden har aktiv tjener s sker den i basen sin (og videre 
	 * til andre tjenere om det ikke er nok sketreff) leverer tilbake. 
	 * 
	 * @param sokestring strengen det skal skes etter.
	 * @return ArrayList med skeordene i String format
	 */
	public ArrayList sokEtter(String sokestring, int teller){
		
		if(this.server.erAktiv()){
			//System.out.println("Fant serveren");
			// System.out.println("Sker n i denne severen: " + this.myName);
			ArrayList tarray = new ArrayList();
			tarray = this.server.sokEtterOrd(sokestring);
			teller++;
			String stringteller = ""+teller;
			tarray.add(0,stringteller);
			return tarray; 
		}
		else{
			// System.out.println("Rekursivt sk");
			int tempteller = teller;
			tempteller++;
			return this.pathNodeToServer.sokEtter(sokestring, tempteller);
			
		}
		
	}
	
}

////////////////////////////////////////////////////////////////////////
//// Metoder som ikke brukes lenger! 
////////////////////////////////////////////////////////////////////////



